@@ -36,6 +36,7 @@ from prnt import cups
 from base.codes import *
 from ui_utils import *
 import hpmudext
+from installer.core_install import *
 
 # Qt
 from PyQt4.QtCore import *
@@ -150,6 +151,18 @@ class PluginInstall(QObject):
         return qApp.translate("DevMgr5",s,c)
 
 
+class DiagnoseQueue(QObject):
+    def __init__(self, parent):
+        self.parent = parent
+
+
+    def exec_(self):
+        ok, output = utils.run('hp-diagnose_queues -r')
+
+    def __tr(self,s,c = None):
+        return qApp.translate("DevMgr5",s,c)
+
+
 
 # ***********************************************************************************
 #
@@ -166,8 +179,7 @@ class DevMgr5(QMainWindow,  Ui_MainWindow):
         log.debug("Initializing toolbox UI (Qt4)...")
         log.debug("HPLIP Version: %s" % prop.installed_version)
 
-        self.setupUi(self)
-
+        
         self.toolbox_version = toolbox_version
         self.initial_device_uri = initial_device_uri
         self.device_vars = {}
@@ -177,13 +189,23 @@ class DevMgr5(QMainWindow,  Ui_MainWindow):
         self.updating = False
         self.init_failed = False
         self.service = None
+        self.Is_autoInstaller_distro = False		# True-->tier1(supports auto installation).   		False--> tier2(manual installation)
 
+        # Distro insformation
+        core =  CoreInstall(MODE_CHECK)
+#        core.init()
+        self.Is_autoInstaller_distro = core.is_auto_installer_support()
         # User settings
         self.user_settings = UserSettings()
         self.user_settings.load()
         self.user_settings.debug()
         self.cur_device_uri = self.user_settings.last_used_device_uri
-
+        installed_version=sys_conf.get('hplip','version')
+        if not utils.Is_HPLIP_older_version( installed_version,  self.user_settings.latest_available_version):
+            self.setupUi(self,"",self.Is_autoInstaller_distro)
+        else:
+            self.setupUi(self, self.user_settings.latest_available_version,self.Is_autoInstaller_distro)
+            
         # Other initialization
         self.initDBus()
         self.initPixmaps()
@@ -278,6 +300,7 @@ class DevMgr5(QMainWindow,  Ui_MainWindow):
         self.initPrintSettingsTab()
         self.initPrintControlTab()
 
+
         self.connect(self.Tabs,SIGNAL("currentChanged(int)"),self.Tabs_currentChanged)
 
         # Resize the splitter so that the device list starts as a single column
@@ -295,6 +318,7 @@ class DevMgr5(QMainWindow,  Ui_MainWindow):
                           2: self.updateSuppliesTab,
                           3: self.updatePrintSettingsTab,
                           4: self.updatePrintControlTab,
+                          5:self.updateHPLIPupgrade,
                         }
 
         # docs
@@ -394,7 +418,7 @@ class DevMgr5(QMainWindow,  Ui_MainWindow):
 
             elif event.event_code == EVENT_SYSTEMTRAY_EXIT:
                 log.debug("EVENT_SYSTEMTRAY_EXIT")
-                log.error("HPLIP Status Service was closed. HPLIP Device Manager will now exit.")
+                log.warn("HPLIP Status Service was closed. HPLIP Device Manager will now exit.")
                 self.close()
 
             elif event.event_code == EVENT_RAISE_DEVICE_MANAGER:
@@ -643,8 +667,9 @@ class DevMgr5(QMainWindow,  Ui_MainWindow):
                     device_list[d] = dev
 
                 log.debug("Removals (2): %s" % ','.join(removals))
-
+                removed_device=None
                 for d in removals:
+                    removed_device = d
                     index = self.DeviceList.count()-1
                     item = self.DeviceList.item(index)
                     log.debug("removing: %s" % d)
@@ -666,7 +691,10 @@ class DevMgr5(QMainWindow,  Ui_MainWindow):
 
                 self.DeviceList.updateGeometry()
                 qApp.processEvents()
-
+                # sending Event to remove this device from hp-systray
+                if removed_device:
+                    utils.sendEvent(EVENT_CUPS_QUEUES_CHANGED,removed_device, "")
+                
                 if len(device_list):
                     for tab in self.TabIndex:
                         self.Tabs.setTabEnabled(tab, True)
@@ -1143,6 +1171,13 @@ class DevMgr5(QMainWindow,  Ui_MainWindow):
                     "plugin",
                     x,
                     lambda : PluginInstall(self, d.plugin, plugin_installed)),
+                    
+                    # Diagnose Queues
+                    (lambda : True,
+                    self.__tr("Diagnose Queues"),
+                    "warning",
+                    self.__tr("Diagnose Print/Fax Queues."),
+                    lambda : DiagnoseQueue(self)),
 
                     # EWS
 
@@ -1446,26 +1481,21 @@ class DevMgr5(QMainWindow,  Ui_MainWindow):
                 self.cur_device.status_type != STATUS_TYPE_NONE and \
                 self.cur_device.device_state != DEVICE_STATE_NOT_FOUND:
 
-                try:
-                    self.cur_device.sorted_supplies
-                except AttributeError:
-                    self.cur_device.sorted_supplies = []
+                self.cur_device.sorted_supplies = []
+                a = 1
+                while True:
+                    try:
+                        agent_type = int(self.cur_device.dq['agent%d-type' % a])
+                        agent_kind = int(self.cur_device.dq['agent%d-kind' % a])
+                        agent_sku = self.cur_device.dq['agent%d-sku' % a]
+                    except KeyError:
+                        break
+                    else:
+                        self.cur_device.sorted_supplies.append((a, agent_kind, agent_type, agent_sku))
 
-                if not self.cur_device.sorted_supplies:
-                    a = 1
-                    while True:
-                        try:
-                            agent_type = int(self.cur_device.dq['agent%d-type' % a])
-                            agent_kind = int(self.cur_device.dq['agent%d-kind' % a])
-                            agent_sku = self.cur_device.dq['agent%d-sku' % a]
-                        except KeyError:
-                            break
-                        else:
-                            self.cur_device.sorted_supplies.append((a, agent_kind, agent_type, agent_sku))
+                    a += 1
 
-                        a += 1
-
-                    self.cur_device.sorted_supplies.sort(lambda x, y: cmp(x[1], y[1]) or cmp(x[3], y[3]))
+                self.cur_device.sorted_supplies.sort(lambda x, y: cmp(x[1], y[1]) or cmp(x[3], y[3]))
 
                 self.SuppliesTable.setRowCount(len(self.cur_device.sorted_supplies))
                 self.SuppliesTable.setColumnCount(len(self.supplies_headers))
@@ -1739,6 +1769,34 @@ class DevMgr5(QMainWindow,  Ui_MainWindow):
         # TODO: Check queues at startup and send events if stopped or rejecting
 
 
+    def initUpgradeTab(self):
+        self.connect(self.InstallLatestButton, SIGNAL("clicked()"), self.InstallLatestButton_clicked)
+        self.InstallLatestButton_lock = False
+        
+       
+    def InstallLatestButton_clicked(self):
+        if self.InstallLatestButton_lock is True:
+            return
+        if self.Is_autoInstaller_distro:
+            self.InstallLatestButton.setEnabled(False)
+            terminal_cmd = utils.get_terminal()
+            if terminal_cmd is not None and utils.which("hp-upgrade"):
+                cmd = terminal_cmd + " 'hp-upgrade'"
+                log.debug("cmd = %s " %cmd)
+                os.system(cmd)
+            else:
+                log.error("Failed to run hp-upgrade command from terminal =%s "%terminal_cmd)
+            self.InstallLatestButton.setEnabled(True)
+        else:
+            self.InstallLatestButton_lock = True
+            utils.openURL("http://hplipopensource.com/hplip-web/install/manual/index.html")
+            QTimer.singleShot(1000, self.InstallLatestButton_unlock)
+
+
+    def InstallLatestButton_unlock(self):
+        self.InstallLatestButton_lock = False
+
+
     def CancelJobButton_clicked(self):
         item = self.JobTable.currentItem()
 
@@ -1752,7 +1810,12 @@ class DevMgr5(QMainWindow,  Ui_MainWindow):
     def RefreshButton_clicked(self):
         self.updatePrintControlTab()
 
+    def  updateHPLIPupgrade(self):
+        self.initUpgradeTab()
 
+        
+        
+        
     def updatePrintControlTab(self):
         if self.cur_device.device_type == DEVICE_TYPE_PRINTER:
             self.PrintControlPrinterNameLabel.setText(self.__tr("Printer Name:"))
